local SEPARATOR = "_"
local REALMLIST = GetCVar("realmList")
local REGION = strsplit(".", REALMLIST)
local MAX_CHARACTERS_PER_SIDE = 40
local inBattleground

-- Binding Variables
BINDING_HEADER_GROUPARMORY_BINDINGS_HEADER = "Grouparmory Companion";
BINDING_NAME_GROUPARMORY_SHOWURL = "Show Grouparmory URL";

---------------------------------------
local function slugify(server)
	---------------------------------------

	if string.len(server) == 0 then
		server = GetRealmName();
	end

	--print("- - - - - - - - - -");
	--print("[1] " .. server);

	local new_server = "";
	for i = 1, server:len() do
		local char = string.sub(server, i, i)
		local prev_char
		if i ~= 0 then
			prev_char = string.sub(server, i - 1, i - 1)
		end
		if i ~= 1 and char ~= "'" and char:upper() == char and prev_char ~= "'" and not char:match("%d") then
			new_server = new_server .. " " .. char
		else
			new_server = new_server .. char
		end
	end
	server = new_server
	--print("[2] " .. server);

	server = server:gsub("'", "");
	--print("[3] " .. server);

	server = server:gsub(" ", "-");
	--print("[4] " .. server);

	-- Handle Area52
	match = server:match("%d+")
	if match then
		server = server:gsub("%d+", "-" .. server:match("%d+"))
	end

	--print("[5] " .. server);

	-- Handle Special Cases
	if server == "Altarof-Storms" then
		server = "Altar-of-Storms"
	end

	--print("[6] " .. server);

	return server
end

---------------------------------------
local function ParseName(identifier)
	---------------------------------------

	local name, server = strmatch(identifier, "^([^%-]+)-?(.*)")
	server = slugify(server);
	return name, server
end

---------------------------------------
local function ShowPopup(url)
	---------------------------------------

	StaticPopupDialogs["POPUP_GROUPARMORY_URL"] = {
		text = "Grouparmory\n\nCopy this URL and paste it into your web browser",
		button1 = "OK",
		OnShow = function(self, data)
			self.editBox:SetText(url);
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		hasEditBox = true,
		preferredIndex = 3, -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
	};
	StaticPopup_Show("POPUP_GROUPARMORY_URL");
end

---------------------------------------
local function BuildListRaid()
	---------------------------------------

	local players = {}
	for i = 1, GetNumGroupMembers() do
		local identifier, _, _, level, class, classToken, _, _, _, role, _ = GetRaidRosterInfo(i);
		local name, server = ParseName(identifier)
		local player = {
			["name"] = name,
			["server"] = server,
		}
		tinsert(players, player)
	end

	local n = #players
	return players, n
end

---------------------------------------
local function DisplaySummaryRaid()
	---------------------------------------

	local left = "";
	local right = "";

	local players, n = BuildListRaid()
	if not players then
		return
	end
	local counter = 0;
	local left_counter = 0;
	local right_counter = 0;
	for i = 1, n do
		if counter % 2 == 0 then
			if left_counter < MAX_CHARACTERS_PER_SIDE then
				if string.len(left) ~= 0 then
					left = left .. ","
				end
				left = left .. REGION .. SEPARATOR .. players[i].server .. SEPARATOR .. players[i].name
				left_counter = left_counter + 1;
			end
		elseif counter % 2 == 1 then
			if right_counter < MAX_CHARACTERS_PER_SIDE then
				if string.len(right) ~= 0 then
					right = right .. ","
				end
				right = right .. REGION .. SEPARATOR .. players[i].server .. SEPARATOR .. players[i].name
				right_counter = right_counter + 1;
			end
		end
		counter = counter + 1
	end

	local url = "www.grouparmory.com/view" .. "?left=" .. left .. "&right=" .. right .. "&ref=wow";
	url = url:lower();

	ShowPopup(url);
end

---------------------------------------
local function BuildListBG()
	---------------------------------------

	SetBattlefieldScoreFaction()

	-- Build list of players
	local players = {}
	for i = 1, GetNumBattlefieldScores() do
		local identifier, _, _, _, _, faction, _, class, classToken, _, _, _, _, _, _, talentSpec = GetBattlefieldScore(i)
		local name, server = ParseName(identifier)
		local player = {
			["name"] = name,
			["server"] = server,
			["faction"] = faction,
		}
		tinsert(players, player)
	end

	local n = #players
	return players, n
end

---------------------------------------
local function DisplaySummaryBG()
	---------------------------------------

	local left = "";
	local right = "";

	local players, n = BuildListBG()
	if not players then
		return
	end
	local left_counter = 0;
	local right_counter = 0;
	for i = 1, n do
		if players[i].faction == 0 then
			if left_counter < MAX_CHARACTERS_PER_SIDE then
				if string.len(left) ~= 0 then
					left = left .. ","
				end
				left = left .. REGION .. SEPARATOR .. players[i].server .. SEPARATOR .. players[i].name
				left_counter = left_counter + 1;
			end
		elseif players[i].faction == 1 then
			if right_counter < MAX_CHARACTERS_PER_SIDE then
				if string.len(right) ~= 0 then
					right = right .. ","
				end
				right = right .. REGION .. SEPARATOR .. players[i].server .. SEPARATOR .. players[i].name
				right_counter = right_counter + 1;
			end
		end
	end

	local url = "www.grouparmory.com/view" .. "?left=" .. left .. "&right=" .. right .. "&ref=wow";
	url = url:lower();

	ShowPopup(url);
end

---------------------------------------
local function showURL()
	---------------------------------------

	if GetNumGroupMembers() == 0 then
		print("Grouparmory Companion | You need to be in a party or raid");
	elseif inBattleground then
		DisplaySummaryBG()
	else
		DisplaySummaryRaid()
	end

	return
end

---------------------------------------
SLASH_GA1, SLASH_GA2 = "/grouparmory", "/ga"
SlashCmdList.GA = function(msg)
---------------------------------------

	showURL();
end

local events = {}
---------------------------------------
function events:PLAYER_ENTERING_WORLD()
	---------------------------------------
	inBattleground = false
end

---------------------------------------
function events:PLAYER_ENTERING_BATTLEGROUND()
	---------------------------------------
	inBattleground = true
end

local frame = CreateFrame("frame")
---------------------------------------
frame:SetScript("OnEvent", function(self, event, ...)
---------------------------------------
	events[event](self, ...)
end)
for k in pairs(events) do
	frame:RegisterEvent(k)
end

---------------------------------------
function GROUPARMORY_SHOWURL_BINDING_FUNCTION()
	---------------------------------------
	showURL();
end
